@@ -186,7 +186,9 @@ class JunosDevice(linux.LinuxDevice):
             tnp_addr: tnp address to execute, if not execut the command on
                       every SPU.
         '''
-        spus = kwargs.get("tnp_addr", None) and [kwargs["tnp_addr"]] or self["spus"]
+        spus = kwargs.get("tnp_addr", None) and kwargs["tnp_addr"] or self["spus"]
+        if type(spus)!=list:
+            spus = [spus]
 
         linux.LinuxDevice.cmd(self, "run start shell csh",
                 expect = self.original_prompt, timeout = 3, with_log = False)
