@@ -108,7 +108,8 @@ default = {
                              # Use self.test_color() to see what each color
                              # looks like
     "log_time":   True,      # record the log with local time;
-    #"log_thread": True,      # record the log with thread name;
+    "LANG":       os.environ.get('LANG'),
+    #"log_thread": True,     # record the log with thread name;
 
     # User can also define Whatever attributes you want.
     # ...
@@ -118,7 +119,7 @@ class LinuxDevice(object):
     '''
     A base class for common linux devices.
     '''
-    release = '1.1.36'
+    release = '1.2.1'
 
     LOG_EMERG = 0   # not used
     LOG_ALERT = 1   # not used
@@ -152,7 +153,7 @@ class LinuxDevice(object):
         self.attributes = collections.OrderedDict()
         self.interface = collections.OrderedDict()
         self.original_prompt = r"[#$%>]" # Used for the first match after login
-        self.prompt = "\[NETDEVICE\][\$\#] "
+        self.prompt = ["\[NETDEVICE\][\$\#] "]
         self.connected = False
         self.random = random.randint(2, len(color_dict) - 5)
 
@@ -239,12 +240,15 @@ class LinuxDevice(object):
         called. Set attribute "autologin" as False and you must call it
         explictly.
         '''
-        cmd = '%s -l %s %s' %(self["scheme"], self["username"], self["hostname"])
-        if self["port"]:
-            if self["scheme"] == 'ssh':
-                cmd += ' -p'
-            cmd += ' %d' %(int(self["port"]))
-        self.log("%s\n" %(cmd))
+        if self["scheme"]  and self["username"] and self["hostname"]:
+            cmd = '%s -l %s %s' %(self["scheme"], self["username"], self["hostname"])
+            if self["port"]:
+                if self["scheme"] == 'ssh':
+                    cmd += ' -p'
+                cmd += ' %d' %(int(self["port"]))
+            self.log("%s\n" %(cmd))
+        else:
+            cmd = 'sh'
         psession = pexpect.spawn("%s" %(cmd))
         self.psession = psession
         if self["log_level"] >= LOG_VERBOSE:
@@ -292,8 +296,10 @@ class LinuxDevice(object):
 
         # need root privilage to configure the interface
         if self["username"] != "root" and self.attributes.get("root_password"):
-            if LinuxDevice.cmd(self, 'su', expect = '(?i)password:', format = "status"):
-                LinuxDevice.cmd(self, self["root_password"], expect = self.original_prompt, log_level = LOG_DEBUG)
+            if LinuxDevice.cmd(self, 'su', expect = '(?i)password:',
+                    format = "status"):
+                LinuxDevice.cmd(self, self["root_password"],
+                        expect = self.original_prompt, log_level = LOG_DEBUG)
 
         # SUBTLE HACK ALERT! Note that the command to set the prompt uses a
         # slightly different string than the regular expression to match it. This
@@ -307,10 +313,11 @@ class LinuxDevice(object):
         # if you 'su' to a different user then you will need to manually reset
         # the prompt. This sends shell commands to the remote host to set the
         # prompt, so this assumes the remote host is ready to receive commands. 
-        if not LinuxDevice.cmd(self, self.PROMPT_SET_CSH, format = "status",
+        if not LinuxDevice.cmd(self, self.PROMPT_SET_SH, format = "status",
                 timeout=1, with_log = False, log_level = self.LOG_DEBUG):
-            LinuxDevice.cmd(self, self.PROMPT_SET_SH, format = "status",
-                                timeout=1, with_log = False, log_level = self.LOG_DEBUG)
+            LinuxDevice.cmd(self, self.PROMPT_SET_CSH, format = "status",
+                                timeout=1, with_log = False,
+                                log_level = self.LOG_DEBUG)
         self.connected = True
         return psession
 
@@ -322,7 +329,7 @@ class LinuxDevice(object):
         del self.psession
         self.psession = self.login()
 
-    def cmd(self, cmd, expect = None, timeout = 20, background = False,
+    def __cmd(self, cmd, expect = None, timeout = 20, background = False,
             logfile = "/dev/null", control = False,
             format = "output", command_leading = "\033[0;31m$ \033[0m",
             command_color = "no_color", command_bg_color = "no_color",
@@ -388,7 +395,7 @@ class LinuxDevice(object):
 
         prompt = [pexpect.EOF, pexpect.TIMEOUT]
         if not expect:
-            prompt.append(self.prompt)
+            prompt.extend(self.prompt)
         else:
             if isinstance(expect, list):
                 prompt.extend(expect)
@@ -399,7 +406,10 @@ class LinuxDevice(object):
             cmd = cmd + ' > %s 2>&1 &' %(logfile)
 
         # Eat the prompt before send line.
-        self.psession.expect(prompt, 0.1)
+        i = 2
+        while i == 2:
+            #self.log("cmd: %s, i: %d, prompt: %s.\n" %(cmd, i, prompt))
+            i = self.psession.expect(prompt, 0.01)
         try:
             if control:
                 self.psession.sendcontrol(cmd)
@@ -411,7 +421,7 @@ class LinuxDevice(object):
             del self.psession
             self.psession = self.login()
             # Eat the prompt before send line.
-            i = self.psession.expect(prompt, 1)
+            i = self.psession.expect(prompt, 0.01)
             self.log("relogin match: %s\n" %(prompt[i]), level = LOG_DEBUG, **kwargs)
             self.log('succeed to re-connect to %s: %d\n' %(self.name, i), **kwargs)
             # resend the command
@@ -428,6 +438,7 @@ class LinuxDevice(object):
                 bg_color = command_bg_color,
                 **kwargs)
         i = self.psession.expect(prompt, timeout)
+        #self.log("cmd(2), i: %d\n" %(i))
 
         status = (i >= 2) and True or False
         if self["log_level"] >= LOG_DEBUG:
@@ -441,8 +452,11 @@ class LinuxDevice(object):
                     color = "light_red",
                     **kwargs)
             #self.psession.sendeof()
-            #self.psession.sendcontrol('c')
+            self.psession.sendcontrol('c')
             #self.psession.sendcontrol('d')
+            i = self.psession.expect(" ", 1)
+            i = self.psession.expect(prompt, timeout)
+            #self.log("cmd(3), i: %d\n" %(i))
 
         # Prepare the output. Strip the line beofre the first '\r\n" since it
         # must be a command itself. Will correct it if encounter issue.
@@ -469,6 +483,7 @@ class LinuxDevice(object):
                     leading = "  ",
                     #color = output_color,
                     color = "dark_gray",
+                    bg_color = output_bg_color,
                     **kwargs)
 
         if format == "output":
@@ -478,6 +493,20 @@ class LinuxDevice(object):
         elif format == "both":
             return (status, output)
 
+    # mutile command in one function:
+    def cmd (self, *args, **kwargs):
+        '''
+        Execute a configure command and return the result of the last command.
+        Sematics is like self.cli, see detail in self.cli()
+        '''
+        result = ""
+        for cmd in args:
+            for c in StringIO(cmd.strip()).readlines():
+                if c.strip() and c.strip()[0] != '#':
+                    result = self.__cmd(c.strip(), **kwargs)
+                    #print("result: %s" %(result))
+        return result
+
     def thread(self, group=None, target=None, name=None, args=(), kwargs={}):
         '''
         To be implemented.
@@ -1143,6 +1172,53 @@ class LinuxDevice(object):
                 return 4
         return 0
 
+    def ip_address_add (self, name, ip = None, ip6 = None):
+        """
+        ip address add "ip" dev "name"
+        """
+        #ip = ipaddress.ip_address(address)
+        self.cmd("ip address flush dev %s" %(name))
+        if ip:
+            self.cmd("ip address add %s dev %s" %(ip, name))
+        if ip6:
+            self.cmd("ip -6 address add %s dev %s" %(ip6, name))
+        self.cmd("ip link set %s up" %(name))
+
+    def ip_address_show (self, ifname, family = 4, host = None, hosttype = None):
+        """
+        ip show ifname on device host(type: hosttype)
+        """
+        if host and hosttype == "kvm":
+            if family == 4:
+                ip = self.cmd("ssh %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+            else:
+                ip = self.cmd("ssh %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+        elif host and hosttype == "netns":
+            if family == 4:
+                ip = self.cmd("ip netns exec %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+            else:
+                ip = self.cmd("ip netns exec %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+        elif host and hosttype == "container":
+            if family == 4:
+                ip = self.cmd("docker exec -it %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+            else:
+                ip = self.cmd("docker exec -it %s ip -o -4 addr list %s | awk '{print $4}'"
+                        %(host, ifname))
+        else: # Get the ip on the linux host
+            if family == 4:
+                ip = self.cmd("ip -o -4 addr list %s | awk '{print $4}'"
+                        %(ifname))
+            else:
+                ip = self.cmd("ip -o -4 addr list %s | awk '{print $4}'"
+                        %(ifname))
+        #return ipaddress.ip_interface(ip.strip()).ip
+        return ip.split("\n")[0]
+
 if __name__ == '__main__':
     pc1 = LinuxDevice("ssh://root:Embe1mpls@skater.englab.juniper.net")
     print(pc1.cmd("ifconfig"))
